class DateTime

  # @private
  SYM_365_day = "365_day".intern
  # @private
  SYM_366_day = "366_day".intern
  # @private
  SYM_360_day = "360_day".intern

  # Parses the given datetime expression and creates an instance.
  # `DateTime._parse()` is called internally.
  #
  # @param spec [String]
  #
  # @return [DateTimeFixedDPY]

  def self.parse_timestamp (spec, format: nil, offset: nil, calendar: "standard", tz: nil)
    case calendar.downcase.intern
    when :standard, :gregorian
      klass = DateTime
      start = Date::ITALY
    when :proleptic_gregorian
      klass = DateTime
      start = Date::GREGORIAN
    when :proleptic_julian, :julian
      klass = DateTime
      start = Date::JULIAN
    when :noleap, SYM_365_day
      klass = DateTime::NoLeap
      start = nil
    when :allleap, SYM_366_day
      klass = DateTime::AllLeap
      start = nil
    when SYM_360_day
      klass = DateTime::Fixed360Day
      start = nil
    end
    if format
      hash = DateTime._strptime(spec, format)
      raise "date-time string '#{spec}' doesn't match with the given format '#{format}'" unless hash
    else
      if spec =~ /\A([+\-]?\d{1,4})(\-(\d{1,2}))?(\s+(\w{3}|[+\-]\d{1,2}(:?\d{1,2})))?\z/
        year  = $1.to_i
        month = $3 ? $3.to_i : 1
        mday  = 1
        rest  = $4
        hash = DateTime._parse("#{year}-#{month}-#{mday} 00:00:00 #{rest}")
      else
        hash = DateTime._parse(spec)
      end
    end
    year, month, day, hour, minute, second, sec_fraction, offset_ = 
           hash.values_at(:year, :mon, :mday, :hour, :min, :sec, :sec_fraction, :offset)
    hour   ||= 0
    minute ||= 0
    second ||= 0.0
    sec_fraction ||= 0.0
    if offset_ 
      offset = offset_.quo(86400)
    else
      offset ||= 0
    end
    if tz
      time = tz.local_datetime(year, month, day, hour, minute, second.to_i, sec_fraction.to_r)
    else
      if hour == 24 && minute == 0 && second == 0.0
        time = klass.new(year, month, day, 23, minute, second + sec_fraction, offset, start) + 1.quo(24)
      else
        time = klass.new(year, month, day, hour, minute, second + sec_fraction, offset, start)
      end    
    end
    return time
  end

end