#
# format.rb is modified version of date/format.rb distributed in Ruby 1.8.7.
#
# format.rb: Written by Tadayoshi Funaba 1999-2008
# $Id: format.rb,v 2.43 2008-01-17 20:16:31+09 tadf Exp $


#
#
#
class DateTimeLike

  # @private
  MILLISECONDS_IN_DAY    = Rational(1, 86400*10**3) 

  # @private
  MILLISECONDS_IN_SECOND = Rational(1, 10**3) 

  # @private
  NANOSECONDS_IN_DAY     = Rational(1, 86400*10**9) 

  # @private
  NANOSECONDS_IN_SECOND  = Rational(1, 10**9) 

  # @private
  SECONDS_IN_DAY         = Rational(1, 86400) 

  # @private
  # Full month names, in English.  Months count from 1 to 12; a
  # month's numerical representation indexed into this array
  # gives the name of that month (hence the first element is nil).
  MONTHNAMES = [nil] + %w(January February March April May June July
			  August September October November December)

  # @private
  # Full names of days of the week, in English.  Days of the week
  # count from 0 to 6 (except in the commercial week); a day's numerical
  # representation indexed into this array gives the name of that day.
  DAYNAMES = %w(Sunday Monday Tuesday Wednesday Thursday Friday Saturday)

  # @private
  # Abbreviated month names, in English.
  ABBR_MONTHNAMES = [nil] + %w(Jan Feb Mar Apr May Jun
			       Jul Aug Sep Oct Nov Dec)

  # @private
  # Abbreviated day names, in English.
  ABBR_DAYNAMES = %w(Sun Mon Tue Wed Thu Fri Sat)

  [MONTHNAMES, DAYNAMES, ABBR_MONTHNAMES, ABBR_DAYNAMES].each do |xs|
    xs.each{|x| x.freeze unless x.nil?}.freeze
  end

  def emit(e, f) # :nodoc:
    case e
    when Numeric
      sign = %w(+ + -)[e <=> 0]
      e = e.abs
    end

    s = e.to_s

    if f[:s] && f[:p] == '0'
      f[:w] -= 1
    end

    if f[:s] && f[:p] == "\s"
      s[0,0] = sign
    end

    if f[:p] != '-'
      s = s.rjust(f[:w], f[:p])
    end

    if f[:s] && f[:p] != "\s"
      s[0,0] = sign
    end

    s = s.upcase if f[:u]
    s = s.downcase if f[:d]
    s
  end

  def emit_w(e, w, f) # :nodoc:
    f[:w] = [f[:w], w].compact.max
    emit(e, f)
  end

  def emit_n(e, w, f) # :nodoc:
    f[:p] ||= '0'
    emit_w(e, w, f)
  end

  def emit_sn(e, w, f) # :nodoc:
    if e < 0
      w += 1
      f[:s] = true
    end
    emit_n(e, w, f)
  end

  def emit_z(e, w, f) # :nodoc:
    w += 1
    f[:s] = true
    emit_n(e, w, f)
  end

  def emit_a(e, w, f) # :nodoc:
    f[:p] ||= "\s"
    emit_w(e, w, f)
  end

  def emit_ad(e, w, f) # :nodoc:
    if f[:x]
      f[:u] = true
      f[:d] = false
    end
    emit_a(e, w, f)
  end

  def emit_au(e, w, f) # :nodoc:
    if f[:x]
      f[:u] = false
      f[:d] = true
    end
    emit_a(e, w, f)
  end

  private :emit, :emit_w, :emit_n, :emit_sn, :emit_z,
          :emit_a, :emit_ad, :emit_au

  def strftime(fmt='%F')
    
    fmt.gsub(/%([-_0^#]+)?(\d+)?([EO]?(?::{1,3}z|.))/m) do |m|
      f = {}
      a = $&
      s, w, c = $1, $2, $3
      if s
        s.scan(/./) do |k|
          case k
          when '-'; f[:p] = '-'
          when '_'; f[:p] = "\s"
          when '0'; f[:p] = '0'
          when '^'; f[:u] = true
          when '#'; f[:x] = true
          end
        end
      end
      if w
        f[:w] = w.to_i
      end
      case c
      when 'A'; emit_ad(DAYNAMES[wday], 0, f)
      when 'a'; emit_ad(ABBR_DAYNAMES[wday], 0, f)
      when 'B'; emit_ad(MONTHNAMES[month], 0, f)
      when 'b'; emit_ad(ABBR_MONTHNAMES[month], 0, f)
      when 'C', 'EC'; emit_sn((year / 100).floor, 2, f)
      when 'c', 'Ec'; emit_a(strftime('%a %b %e %H:%M:%S %Y'), 0, f)
      when 'D'; emit_a(strftime('%m/%d/%y'), 0, f)
      when 'd', 'Od'; emit_n(day, 2, f)
      when 'e', 'Oe'; emit_a(day, 2, f)
      when 'F'
        if m == '%F'
          format('%.4d-%02d-%02d', year, month, day) # 4p
        else
          emit_a(strftime('%Y-%m-%d'), 0, f)
        end
      when 'G'; emit_sn(cwyear, 4, f)
      when 'g'; emit_n(cwyear % 100, 2, f)
      when 'H', 'OH'; emit_n(hour, 2, f)
      when 'h'; emit_ad(strftime('%b'), 0, f)
      when 'I', 'OI'; emit_n((hour % 12).nonzero? || 12, 2, f)
      when 'j'; emit_n(yday, 3, f)
      when 'k'; emit_a(hour, 2, f)
      when 'L'
        emit_n((sec_fraction / MILLISECONDS_IN_SECOND).round, 3, f)
      when 'l'; emit_a((hour % 12).nonzero? || 12, 2, f)
      when 'M', 'OM'; emit_n(minute, 2, f)
      when 'm', 'Om'; emit_n(month, 2, f)
      when 'N'
        emit_n((sec_fraction / NANOSECONDS_IN_SECOND).round, 9, f)
      when 'n'; "\n"
      when 'P'; emit_ad(strftime('%p').downcase, 0, f)
      when 'p'; emit_au(if hour < 12 then 'AM' else 'PM' end, 0, f)
      when 'Q'
        s = ((ajd - self::class::UNIX_EPOCH_IN_AJD) / MILLISECONDS_IN_DAY).round
        emit_sn(s, 1, f)
      when 'R'; emit_a(strftime('%H:%M'), 0, f)
      when 'r'; emit_a(strftime('%I:%M:%S %p'), 0, f)
      when 'S', 'OS'; emit_n(second.floor, 2, f)
      when 's'
        s = ((ajd - self::class::UNIX_EPOCH_IN_AJD) / SECONDS_IN_DAY).round
        emit_sn(s, 1, f)
      when 'T'
        if m == '%T'
          format('%02d:%02d:%02d', hour, minute, second) # 4p
        else
          emit_a(strftime('%H:%M:%S'), 0, f)
        end
      when 't'; "\t"
      when 'U', 'W', 'OU', 'OW'
        emit_n(if c[-1,1] == 'U' then wnum0 else wnum1 end, 2, f)
      when 'u', 'Ou'; emit_n(cwday, 1, f)
      when 'V', 'OV'; emit_n(cweek, 2, f)
      when 'v'; emit_a(strftime('%e-%b-%Y'), 0, f)
      when 'w', 'Ow'; emit_n(wday, 1, f)
      when 'X', 'EX'; emit_a(strftime('%H:%M:%S'), 0, f)
      when 'x', 'Ex'; emit_a(strftime('%m/%d/%y'), 0, f)
      when 'Y', 'EY'; emit_sn(year, 4, f)
      when 'y', 'Ey', 'Oy'; emit_n(year % 100, 2, f)
      when 'Z'; emit_au(strftime('%:z'), 0, f)
      when /\A(:{0,3})z/
        t = $1.size
        sign = if offset < 0 then -1 else +1 end
        fr = offset.abs
        ss = fr.div(SECONDS_IN_DAY) # 4p
        hh, ss = ss.divmod(3600)
        mm, ss = ss.divmod(60)
        if t == 3
          if    ss.nonzero? then t =  2
          elsif mm.nonzero? then t =  1
          else                   t = -1
          end
        end
        case t
        when -1
          tail = []
          sep = ''
        when 0
          f[:w] -= 2 if f[:w]
          tail = ['%02d' % mm]
          sep = ''
        when 1
          f[:w] -= 3 if f[:w]
          tail = ['%02d' % mm]
          sep = ':'
        when 2
          f[:w] -= 6 if f[:w]
          tail = ['%02d' % mm, '%02d' % ss]
          sep = ':'
        end
        ([emit_z(sign * hh, 2, f)] + tail).join(sep)
      when '%'; emit_a('%', 0, f)
      when '+'; emit_a(strftime('%a %b %e %H:%M:%S %Z %Y'), 0, f)
      else
        a
      end
    end
  end

end