require "timesteps"

class DateTime
  
  def self.parse_grads_time (time_string)
    if time_string.strip =~ /\A(((\d{2})?(:(\d{2}))?Z)?(\d{2}))?(\w{3})(\d{4})\z/i
      hour = $3 || "00"
      min  = $5 || "00"
      day  = $6 || "01"
      mon  = $7
      year = $8
      time = DateTime.parse("#{year}#{mon}#{day} #{hour}:#{min}:00")
    else
      raise "invalid time format"
    end
    return time
  end
  
end

class TimeStep::Range

  # @private
  REGEXP_GRADS_TDEF = /\A(?:TDEF\s+)?(\d+)\s+LINEAR\s+([^\s]*?)\s+([^\s]*?)\s*\z/i
  
  # @private
  GRADS_INCREMENT = {
    "mn" => "minutes",
    "hr" => "hours",
    "dy" => "days",
    "mo" => "months",
    "yr" => "years",
  }

  module GrADSMixin

    def parse_grads_time (time_string)
      return DateTime.parse_grads_time(time_string)
    end

  end
    
  def self.from_grads_tdef (tdef_string)

    if tdef_string.strip =~ REGEXP_GRADS_TDEF
      count = $1.to_i
      time  = $2
      increment = $3
    else
      raise "invalid grads tdef string"
    end

    origin = DateTime.parse_grads_time(time)
    
    increment = increment.sub(/(mn|hr|dy|mo|yr)/i) {|s| GRADS_INCREMENT[s.downcase]}
        
    range = TimeStep.new(increment, since: origin).range(count)
    range.extend GrADSMixin

    return range
  end
  
  
end