require "timesteps"

class TimeStep::Converter
  
  def initialize (reference, name: "reference", calendar: "standard")
    @calendar = calendar
    case reference
    when String
      @reference = TimeStep.new(reference, calendar: calendar)
    when TimeStep
      @reference = reference
    else
      raise ArgumentError, "reference argument '#{reference}' should be time-step or string"
    end
    @pair = {}
    @target = {}
    self[name] = reference
  end
  
  # Append or reset new target time step for the given name.
  #
  def add_timestep (spec, name:)
    @pair[name] = TimeStep::Pair.new(@reference, spec, calendar: @calendar)
    @target[name] = @pair[name].to
    return @pair[name]
  end
  
  # Append or reset new target time step for the given name.
  #
  def []= (name, spec)
    return add_timestep(spec, name: name)
  end

  # Return target time step of the given name.
  #
  def [] (name)
    return @pair[name]
  end
  
  # Relete target of the given name.
  #
  def delete (name)
    @target.delete(name)
    @pair.delete(name)
  end

  # Returns the time represented by the given index as DateTime object
  #
  # @param indices [Numeric,Array<Numeric>]
  #
  # @return [DateTime]
  def time_at (*indices)
    return @reference.time_at(*indices)
  end

  # Converts index refering `from` timestep to index refering `to`timestep.
  #
  def forward (*indices, with_time: nil)
    if with_time
      case with_time
      when String
        hash = {
          "time" => time_at(*indices).map{|t| t.strftime(with_time) }
        }
      else
        hash = {
          "time" => time_at(*indices)        
        }
      end
    else
      hash = {}
    end
    @pair.each do |name, conv|
      hash[name] = conv.forward(*indices)
    end
    return hash
  end

  def range (other, with_time: false)
    indices = @reference.range(other)
    return forward(*indices, with_time: with_time)    
  end

  def index_at (*times, format: nil, with_time: false)
    indices = @reference.index_at(*times, format: format)
    return forward(*indices, with_time: with_time)
  end

end

