
class TimeStep

  module DateTimeExt
  
    refine DateTime do
    
      # Returns time fraction in day units.
      #
      # @return [Rational]
      def fraction ()
        return (60*(60*hour + minute) + second + second_fraction).quo(86400)
      end

      def compare_md (other)
        sday = self.day + self.fraction + self.offset
        oday = other.day + other.fraction + other.offset
        if self.month > other.month
          return 1
        elsif self.month < other.month
          return -1
        else
          if sday > oday
            return 1
          elsif sday < oday
            return -1
          else
            return 0
          end
        end
      end

      def compare_d (other)
        sday = self.day + self.fraction + self.offset
        oday = other.day + other.fraction + other.offset
        if sday > oday
          return 1
        elsif sday < oday
          return -1
        else
          return 0
        end
      end

      # Calculate difference between the object and other object in years.
      #
      # @return [Integer]
      def difference_in_years (other)
        my    = self
        other = other.new_offset(self.offset)
        return my.year - other.year + my.compare_md(other).quo(2)
      end
      
      # Calculate difference between the object and other object in months.
      #
      # @return [Integer]
      def difference_in_months (other)
        my    = self
        other = other.new_offset(self.offset)
        return 12*(my.year - other.year) + my.month - other.month + my.compare_d(other).quo(2)  
      end
      
    end

  end
  
end


